@@ -1,6 +1,7 @@
 #include <sys/time.h>
 #include <unistd.h>
 #include <pthread.h>
+#include <_types/_uint64_t.h>
 
 #include "parser.h"
 #include "logging.h"
@@ -49,6 +50,11 @@ static void timeout_flows(struct parser_base *p, struct collector_info *info) {
     struct flow_id *id;
     int ret;
 
+    uint64_t total_bytes = 0;
+    size_t largest_flow_size = 0;
+    struct flow *largest_inmem_flow;
+    struct flow_id *id_of_largest_inmem_flow;
+
     const uint64_t time = p->last_rx_time;
     kv_reset(p->collector_list);
 
@@ -66,6 +72,14 @@ static void timeout_flows(struct parser_base *p, struct collector_info *info) {
             ERROR(p, "Unable to acquire mutex on flow: %s", strerror(ret));
         }
 
+        size_t flow_size = get_tcp_flow_size_bytes((struct tcp_flow *) flow);
+        total_bytes += flow_size;
+        if (flow_size > largest_flow_size) {
+            largest_inmem_flow = flow;
+            id_of_largest_inmem_flow = id;
+            largest_flow_size = flow_size;
+        }
+
         update_flow_avg(flow);
         if (time > flow->expiry_time) {
             if (!(flow->info.flow_state & FLOW_STATE_ITERATED_OVER)) {
@@ -88,6 +102,14 @@ static void timeout_flows(struct parser_base *p, struct collector_info *info) {
             ERROR(p, "Unable to acquire mutex on flow: %s", strerror(ret));
         }
 
+        size_t flow_size = get_udp_flow_size_bytes((struct udp_flow *) flow);
+        total_bytes += flow_size;
+        if (flow_size > largest_flow_size) {
+            largest_inmem_flow = flow;
+            id_of_largest_inmem_flow = id;
+            largest_flow_size = flow_size;
+        }
+
         update_flow_avg(flow);
         if (time > flow->expiry_time) {
             if (!(flow->info.flow_state & FLOW_STATE_ITERATED_OVER)) {
@@ -101,6 +123,15 @@ static void timeout_flows(struct parser_base *p, struct collector_info *info) {
         }
     }
 
+    if (p->soft_mem_limit && largest_inmem_flow != NULL) {
+        if (total_bytes > p->soft_mem_limit && 
+                !(largest_inmem_flow->info.flow_state & FLOW_STATE_ITERATED_OVER)) {
+            largest_inmem_flow->info.flow_state |= FLOW_STATE_PASSIVE_TIMED_OUT;
+            kv_push(struct flow_id *, p->collector_list, id_of_largest_inmem_flow);
+            ERROR(p, "Forcibly offloaded flow of size %u bytes", largest_flow_size);
+        }
+    }
+
     if ((ret = pthread_rwlock_unlock(&p->flows_table_lock)) != 0) {
         ERROR(p, "Unable to release read lock on flow table: %s", strerror(ret));
         return;
