@@ -211,11 +211,14 @@ int close_sniff(struct parser_base *p) {
     }
 
     if (p->sniffer_status == RUNNING) {
-        INFO(p, "Joining sniffer thread");
-        if ((ret = pthread_join(p->sniffer, &status)) != 0) {
-            ERROR(p, "Unable to join sniffer thread: %s", strerror(ret));
-            pthread_mutex_unlock(&p->thread_status_mutex);
-            return -1;
+        pthread_t current_thread = pthread_self();
+        if (!pthread_equal(current_thread, p->sniffer)) {
+            INFO(p, "Joining sniffer thread");
+            if ((ret = pthread_join(p->sniffer, &status)) != 0) {
+                ERROR(p, "Unable to join sniffer thread: %s", strerror(ret));
+                pthread_mutex_unlock(&p->thread_status_mutex);
+                return -1;
+            }
         }
         
         p->sniffer_status = KILLED;
@@ -237,13 +240,13 @@ int init_sniff(struct parser_base *p, struct sniff_data *init_data) {
         ERROR(p, "Unable to lock thread status mutex: %s", strerror(ret));
         return -1;
     }
-    
+
     if (p->sniffer_status != NOT_STARTED) {
         ERROR(p, "Tried to init sniffer twice");
         pthread_mutex_unlock(&p->thread_status_mutex);
         return -1;
     }
-    
+
     memcpy(&p->sniffer_data, init_data, sizeof (struct sniff_data));
 
     if (sniff_open(p) < 0) {
@@ -278,12 +281,12 @@ int init_sniff(struct parser_base *p, struct sniff_data *init_data) {
     }
 
     p->sniffer_status = RUNNING;
-    
+
     if ((ret = pthread_mutex_unlock(&p->thread_status_mutex)) != 0) {
         ERROR(p, "Unable to unlock thread status mutex: %s", strerror(ret));
         return -1;
     }
-    
+
     return 0;
 }
 
